<?php
/**
 *
 * Class for mapping SmartBill Products
 *
 * @link       http://www.smartbill.ro
 * @since      1.0.0
 *
 * @copyright  Intelligent IT SRL 2018
 * @package    smartbill-facturare-si-gestiune
 * @subpackage smartbill/classes
 */

/**
 * Class responsible for managing smartbill products
 *
 * @since      3.1.0
 * @copyright  Intelligent IT SRL 2018
 * @package    smartbill-facturare-si-gestiune
 * @subpackage smartbill/classes
 * @author     Intelligent IT SRL <vreauapi@smartbill.ro>
 */
class SmartbillProduct {
    /**
	 * The Name of the SmartBill product.
	 *
	 * @since    3.1.0
	 * @access   private
	 * @var      string    $name    The Name of the SmartBill product.
	 */
    private $name; 
    
    /**
	 * The sku code of the SmartBill product.
	 *
	 * @since    3.1.0
	 * @access   private
	 * @var      string    $sku    The sku code of the SmartBill product.
	 */
    private $sku;

    /**
	 * The measuring unit of the SmartBill product.
	 *
	 * @since    3.1.0
	 * @access   private
	 * @var      string    $measuring_unit    The measuring unit of the SmartBill product.
	 */
    private $measuring_unit;

    /**
	 * The quantity of the SmartBill product.
	 *
	 * @since    3.1.0
	 * @access   private
	 * @var      int    $quantity    The quantity of the SmartBill product.
	 */
    private $quantity;

    /**
	 * The Prestashop product that coresponds to the SmartBill product.
	 *
	 * @since    3.1.0
	 * @access   private
	 * @var      Product|null    $ps_product    Prestashop product.
	 */
    private $ps_product=null;

    /**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.1.0
	 * @param    string    $name              The Name of the SmartBill product.
     * @param    string    $sku               The sku code of the SmartBill product.
     * @param    string    $measuring_unit    The measuring unit of the SmartBill product.
	 * @param    int       $quantity          The quantity of the SmartBill product.
	 */
    public function __construct($n, $c, $mu, $q){
        $this->name           = (string) $n;
        $this->sku            = (string) $c;
        $this->measuring_unit = (string) $mu;
        $this->quantity       = (int) $q;
    }

    /**
	 * Find prestashop product either by sku or by name.
	 * 
     * @param    string    $sku_type           The code type for product search.
     * 
     * @return Array $ps_product Prestashop product id and combination id 
	 */
    private function find_ps_product_data($sku_type){
        if(!empty( $this->sku )){
            $this->ps_product = $this->get_product_stocks_data_by_ref($this->sku,$sku_type,$this->name);
        }else{
            $this->ps_product = $this->get_product_stocks_data_by_name($this->name,$sku_type);
        }

        return $this->ps_product;
    }

   	/**
	 * Update prestashop product stock if possible.
	 *
	 * @return boolean 
	 */
    public function sync_quantity($sku_type){
        $product_data = $this->find_ps_product_data($sku_type);
        if(empty($product_data[0])){
            return false;
        }

        $product = new Product((int)$product_data[0]);
        StockAvailable::setQuantity((int)$product_data[0],(int) $product_data[1], $this->quantity);
       

        if( false !== $product->update()){
            return true;
        }else{
            return false;
        }

    }   

    /**
	 * Search for prestashop product by name.
	 *
	 * @param string $productName used for searching product by name.
     * @param string $skuType column for database product search.
	 *
	 * @return Product|null $prestashop_product
	 */
    private function get_product_stocks_data_by_name($productName,$skuType){
        $productName=pSQL($productName);
        $skuType=pSQL($skuType);       
        if($skuType != "product_id"){
            //Cauta produs fara cod referinta (ps_product)
            $product_id=Db::getInstance()->getValue('SELECT '._DB_PREFIX_.'product_lang.id_product, '._DB_PREFIX_.'product_lang.name, '._DB_PREFIX_.'product.'.$skuType.' FROM '._DB_PREFIX_.'product INNER JOIN '._DB_PREFIX_.'product_lang ON '._DB_PREFIX_.'product.id_product = '._DB_PREFIX_.'product_lang.id_product WHERE LEFT('._DB_PREFIX_.'product_lang.name,LENGTH('._DB_PREFIX_.'product_lang.name))=LEFT("'.$productName.'",LENGTH('._DB_PREFIX_.'product_lang.name)) AND  '._DB_PREFIX_.'product.'.$skuType.'=""');

            if(!$product_id){
                //Cauta produs cu combinatii fara cod referinta (ps_product_attribute)
                $product_id=Db::getInstance()->getValue('SELECT '._DB_PREFIX_.'product_lang.id_product, '._DB_PREFIX_.'product_lang.name, '._DB_PREFIX_.'product_attribute.'.$skuType.' FROM '._DB_PREFIX_.'product_attribute INNER JOIN '._DB_PREFIX_.'product_lang ON '._DB_PREFIX_.'product_attribute.id_product = '._DB_PREFIX_.'product_lang.id_product WHERE LEFT('._DB_PREFIX_.'product_lang.name,LENGTH('._DB_PREFIX_.'product_lang.name))=LEFT("'.$productName.'",LENGTH('._DB_PREFIX_.'product_lang.name)) AND '._DB_PREFIX_.'product_attribute.'.$skuType.'=""');
            }
        }else{
            $product_id=Db::getInstance()->getValue('SELECT '._DB_PREFIX_.'product_lang.id_product, '._DB_PREFIX_.'product_lang.name FROM '._DB_PREFIX_.'product_attribute INNER JOIN '._DB_PREFIX_.'product_lang ON '._DB_PREFIX_.'product_attribute.id_product = '._DB_PREFIX_.'product_lang.id_product WHERE LEFT('._DB_PREFIX_.'product_lang.name,LENGTH('._DB_PREFIX_.'product_lang.name))=LEFT("'.$productName.'",LENGTH('._DB_PREFIX_.'product_lang.name))');
        }

        $id_attribute=0;
        if($product_id){
            $product_obj = new Product((int) $product_id);
            
            $combinations=$product_obj->getWsCombinations();
            foreach ($combinations as $comb){
                $combination=$product_obj->getAttributeCombinationsById((int)$comb['id'],1);
                $comb_attr=$product_obj->name[1].' -';
                for ($i = 0; $i < count($combination); $i++){
                    $separator = Configuration::get('PS_ATTRIBUTE_ANCHOR_SEPARATOR');
                    if($i!=count($combination)-1){
                        $comb_attr=$comb_attr.' '.trim($combination[$i]['group_name'])." : ".trim($combination[$i]['attribute_name'].$separator);
                    }else{
                        $comb_attr=$comb_attr.' '.trim($combination[$i]['group_name'])." : ".trim($combination[$i]['attribute_name']);
                    }
                }
                if($comb_attr==$productName){
                        $id_attribute=$comb['id'];
                        break;
                }
            }  
        }
        
        return [$product_id, $id_attribute];
    }
    
    /**
	 * Search for prestashop product by code.
	 *
	 * @param string $productCode product sku.
     * @param string $skuType column for database product search.
     * @param string $productName used for searching combinations.
	 *
	 * @return Product|null $prestashop_product
	 */

    private function get_product_stocks_data_by_ref($productCode,$skuType,$productName){
        $productCode=pSQL($productCode);
        $skuType=pSQL($skuType);
        $productName=pSQL($productName);
        $id_attribute=0;

        if($skuType!="product_id"){
            $product_id=Db::getInstance()->getValue('SELECT id_product FROM '._DB_PREFIX_.'product WHERE '.$skuType.' ="'.$productCode.'";');
            
            if($product_id){
                $product_obj = new Product((int) $product_id);
                if($product_obj->hasCombinations()){
                    $product_id_attribute=Db::getInstance()->getValue('SELECT id_product FROM '._DB_PREFIX_.'product_attribute WHERE reference ="'.$productCode.'";');
                    if($product_id_attribute==$product_id){
                        //SKU duplicate found
                        $product_id=null;
                    }
                }
                
            }else{
                $product_id=Db::getInstance()->getValue('SELECT id_product FROM '._DB_PREFIX_.'product_attribute WHERE '.$skuType.' ="'.$productCode.'";');
                $product_obj = new Product((int) $product_id);
                $id_lang = Context::getContext()->language->id;
                $combinations = $product_obj->getAttributeCombinations((int)$id_lang, true);
                foreach ($combinations as $combination){
                    if($combination[$skuType]==$productCode){
                        $id_attribute=$combination['id_product_attribute'];
                        break;
                    }
                }
            }

        }else{
            $product_id=$productCode;
            $product_obj = new Product((int) $product_id);
            
            $combinations=$product_obj->getWsCombinations();
            foreach ($combinations as $comb){
                $combination=$product_obj->getAttributeCombinationsById((int)$comb['id'],1);
                    $comb_attr=$product_obj->name[1].' -';
                    for ($i = 0; $i < count($combination); $i++){
                        $separator = Configuration::get('PS_ATTRIBUTE_ANCHOR_SEPARATOR');
                        if($i!=count($combination)-1){
                            $comb_attr=$comb_attr.' '.trim($combination[$i]['group_name'])." : ".trim($combination[$i]['attribute_name'].$separator);
                        }else{
                            $comb_attr=$comb_attr.' '.trim($combination[$i]['group_name'])." : ".trim($combination[$i]['attribute_name']);
                        }
                    }
                    if($comb_attr==$productName){
                            $id_attribute=$comb['id'];
                            break;
                    }
            }  
        }
      
        return [$product_id, $id_attribute];
       
    }

    /**
	 * Get Object properties as array for use in csv export.
	 *
	 * @return Array $product.
	 */
    public function to_arr(){
        $product = $this->ps_product;
        $product_title="";
        $product_id="";

        if(empty($product[0])){
            $product_title = Context::getContext()->getTranslator()->trans( 'Produsul nu a fost gasit in nomenclatorul Prestashop.', array(), 'Modules.Smartbill.Admin');
        }
        else{
            $product_obj = new Product((int) $product[0]);
            $product_title = $product_obj->name;
            if(is_array($product_title)){
                $lang = new Language( (int) Configuration::get( 'PS_LANG_DEFAULT' ) );
                $product_title=$product_title[$lang->id];
            }
            $product_id = $product[0]; 
        }
    
        return [$this->sku, $this->name, $product_title, $product_id, $this->quantity];
    }
}